#ifndef _EXCHANGE_H
#define _EXCHANGE_H

/** \file exchange.h 
   :     RS485. */

/**         RS485,     .
 ask*, send*  readData           mutex'   - RS485_transaction,   . 
        Error::Exchange.

\note !    ""    ,     
  ,     send*Command()    deviceBusy() 
-   .
*/
namespace RS485 {
    /**     RS485. 
    \param dev RS485 driver special file
        
      \a dev ,   ( ),
          rs485 (com485  lpt  moxa*).    
    (3  ),        -  -     
       YYMMDD.elog.
      - ,  Error::Exchange  ; ,  ,
        - RS_strerror()    errno. */
    void initExchange( const char * dev = "/dev/mcua0" ); 

    /**   \a a ( \a c)      \a b
    \return true -      *b, false -    *b   */
    int askByte( int a, int c, char* b);

    /**   \a a ( \a c)      \a w
    \return true -      *w, false -    *w   */
    int askWord( int a, int c, short* w);

    /**   \a a ( \a c)   \b    \a l.
    \return true - 3      *l, false -    *l  .
    \note  ()   \a l    ,   
    ( )    0xff  <0. */
    int askLong( int a, int c, int* l);

    /**    \a a ( \a c)      MAX_PACKET_SIZE  \a d.
    \return    (0..MAX_PACKET_SIZE)  \b -1   . */
    int  askData( int a, int c, unsigned char* d); 

    static const int DONE = 1; // return value of send*Command when ACY or ACK got
    static const int BUSY = -1; // return value of send*Command when ACW got
    /**    \a c  \a a  
    \return 0 - transaction failed, -1 - got ACW, 1 - got ACY 
    */
    int sendSimpleCommand( int a, int c);

    /**   \a c  \a a  1-  \a b
    \return 0 - transaction failed, -1 - got ACW, 1 - got ACY 
    */
    int sendByteCommand( int a, int c, char b);
    
    /**   \a c  \a a  2-  \a w
    \return 0 - transaction failed, -1 - got ACW, 1 - got ACY 
    */
    int sendWordCommand( int a, int c, short w);

    /**    \a d  \a length  \a a
    \return 0 - transaction failed, -1 - got ACW, 1 - got ACY or ACK 
    */
    int sendData( int a, int length, unsigned char* d); 
    
    /**     \a in  \a length,  ģ  \a out.
    \return 0<return<MAX_PACKET_SIZE:  ,       \a out, 0:  
     ,     \a out. 
    \note  \a out      MAX_PACKET_SIZE . */
    int  sendVoid( unsigned char* in, int length, unsigned char* out); 

    /**       \a d. 
    \param h Data header (4 bytes)
    \param d Data
    \return     \a d. 
    \note  \a d      IO_BUFF_SIZE . */
    int  readData( unsigned char* h, unsigned char* d); 

    /**       unsigned short \a d. 
    \param a Data source address
    \param packnum sequential packet number
    \return   shorts  \a d. 
    \note  \a d      IO_BUFF_SIZE/2 .
    \a packnum is reset in resetData().
    */
    int  readData( int * a, int * packnum, unsigned short* d); 

    /**   
    \param r   kBaud
    */
    
    void setBaudeRate( int r );
    
    /** RS485 driver FIFO clearing */
    void cleanBuffer( void );
    
    /** Prepare to receive new data block (IO and packet number reset) */
    void resetData( void );
}

///          Mount.
namespace Error {
    ///          exchange.
    struct  Exchange {
        const char* from; ///<  "RS485 initialization"  "RS485 exchange".
        const char* what; ///<  
        char why; ///<  ,   (  )
        Exchange( const char* ch, const char* mn, char cs = 0 ) :
            from(ch), what(mn), why(cs) {};
    };
}



#ifdef  _EXCHANGE

#define low(i)      (unsigned char)((i)&0x00ff)
#define high(i)     (unsigned char)(((i)&0xff00)>>8)

const int MAXRETRY = 3; ///<           .
const int IO_BUFF_SIZE = 64; ///<     
const int MAX_PACKET_SIZE = 32; ///<      


#define ACK  SignalACK
#define NAK  SignalNAK
#define NOD  SignalNOD
#define ACN  SignalACN
#define ACY  SignalACY
#define ACW  SignalACW
#define SINC SignalSINC
#define DNG  SignalDNG
#define PKL  SignalPKL
#define PBO  SignalPBO
#define DSC  SignalDSC
#define COL  SignalCOL
#define EER  SignalEER
#define UND  SignalUND

enum { INFO_SIG, LOOP_SIG, CHECK_SIG, ABORT_SIG };

struct Sig {
    int   code;
    int   stype;
    const char* abbr;
    const char* text;
};

#endif
#endif
