#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio_ext.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <time.h>
#include <unistd.h>
#include "exchange.h"
#include "astrotime.h"
#include "version.h"

char	string[128];
char    wsign[16];
char    device[256]="/dev/mcua0";
short   outint[32];
unsigned char buff[32];
unsigned char outstr[64];
const char *vers = LMOX_VERSION_TEXT;
FILE	*inpfile;
unsigned int ipause = 20;
int ibaude_rate = -1;
AstroDateTime utc;
bool iswait = false;
bool decimal = false;

int main( int ac, char **av){
    int nbytes = 0;
    int ii;
    fprintf( stdout, "\n                lmox tool Ver.%s\n", vers );
    opterr = 0;
    while( (ii = getopt(ac, av, ":p:r:w:D:hd")) != -1 ) {
        switch (ii) {
            case 'h':   fprintf( stdout, "Usage: lmox [-p ms] [-r kbaud] [-D device] [file]\n");
                        exit(1);
            case 'w':   if(optarg!=NULL) sscanf( optarg, "%s", wsign );
                        iswait = true;
                        break;
            case 'D':   if(optarg!=NULL) sscanf( optarg, "%s", device );
                        break;
            case 'd':   decimal = true;
                        break;
            case 'p':   if(optarg!=NULL) sscanf( optarg, "%d", &ipause );
                        break;
            case 'r':   if(optarg!=NULL) sscanf( optarg, "%d", &ibaude_rate );
                        break;
            case '?':   fprintf( stdout, "Option -%c is not legal..\n", optopt );
                        exit(1);
            case ':':   fprintf( stdout, "Option -%c has missing argument..\n", optopt );
            default:    exit(1);
        }
    }
    if( av[optind] == NULL ) {
        inpfile = stdin;
    } else {
        inpfile = fopen( av[optind], "r" );
    }

    char * line = 0;
    try{
        RS485::initExchange( device );
        if( ibaude_rate > 0 ) RS485::setBaudeRate( ibaude_rate );
        while(1){
            if( !line ) {
              printf("<");
              if ( fgets( string, 128, inpfile ) == 0 ) break;
              else line = string;
            }
            
            if( strlen( line ) < 4 ) break;
            if( inpfile != stdin ) fprintf( stdout, "%s", line );
            if( line[0] == '#' || line[1] == '#' ) {line = 0; continue;}
            int np = sscanf( line, "%hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx",
                buff,buff+1,buff+2,buff+3,buff+4,buff+5,buff+6,buff+7,buff+8,buff+9 );
            if( np == 0 || np == EOF ) break;

            do nbytes = RS485::sendVoid( buff, np, outstr );
            while( iswait && strcmp( (const char*)outstr, wsign )==0 && !usleep(ipause*1000));
            if( nbytes ) {
                if( !decimal ) {
                    printf("Data %d bytes: ", nbytes );
                    for( int i=0; i < nbytes; i++ ) printf("%02x ", outstr[i] );
                    printf("\n");
                } else {
                    if( nbytes%2 == 0 ) {
                        int nints = nbytes/2;
                        memcpy( (char*)outint, outstr, nbytes );
                        printf("Data %d integers: ", nints );
                        for( int i=0; i < nints; i++ ) printf( "%6d ", outint[i] );
                        printf("\n");
                    }                 
                    if( nbytes == 3 ) {
                        int a = outstr[0]+256*(outstr[1]+256*outstr[2]);
                        if( a > 8388607 ) a = a - 16777216;
                        printf("Data %d \n", a );
                    }
                    if( nbytes == 1 ) {
                        int a = outstr[0];
                        printf("Data %d \n", a );
                    }
                }
            } else  printf("Signal %s\n", outstr );

            char * sc = strchr(line,';');
            if (sc && strlen(sc)>4) {
              line = sc + 1;
            } else {
              usleep(ipause*1000);
              line = 0;
            }
        }
    }
    catch( Error::Exchange err ){
        fprintf( stdout, "%s: %s\n", err.from, err.what );
        exit(FAIL);
    }
    exit(0);
}
